#!/usr/local/bin/perl
###############################################################################
#		Written By:  Torquil Gault (peterb@hotmail.com) C) 2000-
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
###############################################################################
use Text::ParseWords;
use IO::File;

#
###############################################################################
sub createID {
#	$sid++;
	sleep 1;
	$sid = int(time - 957415234);		# A dummy procedure
	return $sid;
}

#
###############################################################################
sub newScheduleItem {
	my $np = @_;
	my $id = 0;
	$obj = new scheduleItem;

	if ($np == 0) {		# Called with no parameters so create a new ID
		$id = &createID();
	}
	else {					# Use the ID passed to it ....
		$id = int($_[0]);
		if ($np == 11) {	# Passed a 'full' structure to it so use that
			$obj->schDate($_[1]);
			$obj->schTime($_[2]);
			$obj->schCycle($_[3]);
			$obj->schCommand($_[4]);
			$obj->schStatus($_[5]);
			$obj->schDependancyID($_[6]);
			$obj->schSpecial($_[7]);
			$obj->schStarted($_[8]);
			$obj->schFinished($_[9]);
			$obj->schRescheduleID($_[10]);
		}
	}
	$obj->schID($id);

	dprint(join(": ", "nSI", $obj->schID, $obj->schDate, $obj->schTime, $obj->schCommand));

	$scheduleItems{$id} = $obj;

	return $id;
}

#
###############################################################################
sub getScheduleItem {
	my $id = $_[0];

	$sItem = $scheduleItems{$id};
	return unless $sItem;
	dprint(join(": ", "gSI", $sItem->schID, $sItem->schDate));
	return $sItem;
}

#
###############################################################################
sub clearScheduleItem {
	my $id = $_[0];
	undef $scheduleItems{$id};
	dprint("cSI: $id");
}

#
###############################################################################
#  Load the schedule into our system
sub loadScheduleFile {
	open(scf, "$Schedule_File") or die "Unable to Load Schedule File ... Bye";
	while(<scf>) {	# loop thro each line and make a scheduleItem
		chop($_);
		@f = quotewords(",", 0, $_);
		&newScheduleItem($f[0],$f[1],$f[2],$f[3],$f[4],$f[5],$f[6],$f[7],$f[8],$f[9],$f[10],);
	}
	close(scf);
}

#
###############################################################################
# Save the schedule to a text file
sub saveScheduleFile {
	open(scf, ">$Schedule_File") or die "Unable to Save Schedule File ... Bye";
	foreach $k(keys(%scheduleItems)) {
		break unless &getScheduleItem($k);
		dprint("sSF: $k");
		print scf $sItem->schID, ",\"", $sItem->schDate, "\",\"", $sItem->schTime, "\",\"",
					 $sItem->schCycle, "\",\"", $sItem->schCommand, "\",", $sItem->schStatus, ",",
					 $sItem->schDependancyID, ",\"", $sItem->schSpecial, "\",\"",
					 $sItem->schStarted, "\",\"", $sItem->schFinished, "\",",
					 $sItem->schRescheduleID;
	}
	close(scf);
}

#
###############################################################################
# Now find the next item to run .....
sub getNextItem {
	&setCurrentDate();
	&setCurrentTime();
	my $d = $m = $y = $t = "";
	foreach my $k(keys(%scheduleItems)) {
		break unless &getScheduleItem($k);
		if ($sItem->schStatus == $Status_Scheduled) {	# Ok, it's ready to run...
			$t = $sItem->schTime;
			$t =~ s?\D??g;			# get a 'clean' time ie. HHMM
			($d,$m,$y) = split($Date_Delimiters, $sItem->schDate);
			$d = join("", $y,$m,$d);	# for easy comparison
			dprint("gNI: $k: $curDate:$d $curTime:$t");
			if ($d < $curDate || ($d == $curDate && $t <= $curTime)) {	# Ok, date/time is ready to run
				if ($sItem->schDependancyID == 0) {		# No dependancies, run it ...
					&runScheduledItem($k);
				}
				else {
					$s = &getItemStatus($sItem->schSpecial, $sItem->schDependancyID);
						# Dependancy has finished or we dont care how it finished
					dprint("gNI: DS = $s");
					if ($s == $Status_Completed 
					 || ($s == $Status_Failed && index($sItem->schSpecial, $Special_Ignore) != -1)) {
						&runScheduledItem($k);
					}
				}
			}
		}
	}
}

#
###############################################################################
# Run the specified item but check to make sure we've enough runqueue first ..
sub runScheduledItem {
	my $e = keys(%runQueue);
	if ($e < $No_Schedulers) {		# Do we have a free run slot ....
		$sItem->schStatus($Status_Running);
		$runQueue{$_[0]} = $_[0];
		$cmd = $sItem->schCommand;
		dprint("rSI: Running $_[0] in $e:$cmd");
		system("runCommand.pl -p $$ -s $SIGQUE[$e] -f $SIGQUE[$e+8] \"$cmd\" &");
	}
}

#
###############################################################################
# Find the status of an item
sub getItemStatus {
	my $si = $scheduleItems{$_[1]};
	dprint("gIS: $_[0] $_[1]");
	if (index($_[0], $Special_Follow) != -1) {	# hmmm ... follow the list down
		return &followItem($_[1]);
	}
	return $si->schStatus;
}

#
###############################################################################
# Follows the ID down it's rescheduled list until it gets to the end ...
sub followItem {
	my $si = $scheduleItems{$_[0]};
	my $i = $ps = 0;
	dprint("fI: $_[0]");
	do {
		$ps = $si->schStatus;
		$i = int($si->schRescheduleID);
		$si = $scheduleItems{$i};
		dprint("fI: $i");
		return unless $si;
	} until ($si->schRescheduleID == 0 || $i == $si->schRescheduleID);
	if ($si->schStatus == $Status_Completed || $si->schStatus == $Status_Failed) {
		return $si->schStatus;
	}		# If the last one hasn't run then return the previous item's status
	return $ps;
}

#
###############################################################################
# Display all the running commands
sub showRunningItems {
	my $si = 0;
	foreach $k(keys(%runQueue)) {
		$si = $scheduleItems{$k};
		return unless $si;
		print "sRI: ", $si->schID, ": ", $si->schCommand, "\n";
	}
}

#
###############################################################################
# Received a signal from a run command so work out which one & it's exit status
sub commandRunStatus {
	my $sig = $@;
	dprint("cRS: $sig received ....");
	$sig = ($sig - $SIGQUE[0]);
	if ($sig > $Max_Schedulers) {	# Hmmm ... it failed
		$sig -= $Max_Schedulers;
	}
	else {
	}
}

#
###############################################################################
sub dummy {
}

return true;
