#!/usr/local/bin/perl
###############################################################################
#		Written By:  Torquil Gault (peterb@hotmail.com) C) 2000-
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
###############################################################################
use Getopt::Std;
use Class::Struct;

###############################################################################
struct('scheduleItem' => {		# The structure of the item
	'schID' => '$',
	'schDate' => '$',
	'schTime' => '$',
	'schCycle' => '$',
	'schCommand' => '$',
	'schStatus' => '$',
	'schDependancyID' => '$',
	'schSpecial' => '$',
	'schStarted' => '$',
	'schFinished' => '$',
	'schRescheduleID' => '$',
});

#%scheduleItems = {};			# This contains all the items, so make an empty list
#%runQueue = {};					# Contains the ID's of all the running items

###############################################################################
$True = 1;
$False = 0;
$Good = $True;
$Bad = $False;

$debug = $True;

$Status_Failed = -1;
$Status_Scheduled = 0;
$Status_Running = 1;
$Status_Completed = 2;

$Special_Ignore = 'I';	# By settting schSpecial to this, it will run no matter the exit status
$Special_Follow = 'F';	# It will verify against the latest run of the schDependancyID

$Schedule_Daily = 'F';
$Schedule_Weekly = 'W';
$Schedule_Monthly = 'M';
$Schedule_Yearly = 'Y';

$Date_Fmt_1 = "+%d/%m/%Y-%H:%M:%S";
$Date_Fmt_2 = "+%d/%m/%Y";
$Time_Fmt_1 = "+%H:%M";

$Date_Delimiters = " |-|/";		# The most common delimiters of a date

$No_Schedulers = 2;		# No of active schedulers/programs at any given time
$Max_Schedulers = 8;		# Max no of schedulers ... hooks into SIGQUE
	# Which signal to send back to the main process to tell it they have finished eg.
	# runQueue 1 would return either 16 or 24 depending on if it succeeded or failed
	# runQueue 2 would return either 17 or 25 depending on if it succeeded or failed and so on
@SIGQUE = (16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,33);
@SIGNAME = (USR1, USR2, CHLD, PWR, VTALRM, PROF, IO, WINCH, STOP, TSTP, CONT, TTIN, TTOU, URG, LOST, XCPU);

$SleepTime = 30;			# No. of seconds that it sleeps between checking

$Scheduler_ExitDate = "/home/peterb/perl/syssched/sched_date.txt";

$Schedule_File = "/home/peterb/perl/syssched/sched_file.txt";
###############################################################################
				# now load all the required functions/procedures etc
require "TimeFuncs.pl";
require "MiscFuncs.pl";
require "SchedFuncs.pl";
require "ScrnFuncs.pl";

require "scConfig.pl";	# load the local config

#
###############################################################################
sub Initialise {
	getopts("d:s:") || die &Usage();
	if ($opt_d) { $debug = $opt_d; }
	if ($opt_s) { $SleepTime = $opt_s; }

	$SIG{'HUP'} = 'ReloadScheduleData';
	$SIG{'INT'} = $SIG{'QUIT'} = $SIG{'TERM'} = 'EndProgram';

	&loadCurrentTime();		# Get the last time it was stopped/exitted

	&setCurrentDate();		# Set the current date vars
	&setCurrentTime();		# Set the current time vars

#	foreach $sig(@SIGNAME) {	# Trap the return status of a run command
#		$SIG{'$sig'} = 'commandRunStatus';
#	}
	$SIG{'USR1'} = $SIG{'USR2'} = $SIG{'CHLD'} = $SIG{'PWR'} = $SIG{'VTALRM'} = 'commandRunStatus';
	$SIG{'PROF'} = $SIG{'IO'} = $SIG{'WINCH'} = $SIG{'STOP'} = $SIG{'TSTP'} = 'commandRunStatus';
	$SIG{'CONT'} = $SIG{'TTIN'} = $SIG{'TTOU'} = $SIG{'URG'} = $SIG{'LOST'} = $SIG{'XCPU'} = 'commandRunStatus';
	if ($No_Schedulers > $Max_Schedulers) {
		$No_Schedulers = $Max_Schedulers;
	}
}

#
###############################################################################
sub Usage {
	print <<EOF;
Usage: $0 [-d debug] [-s sleeptime]
EOF
	exit 0;
}

#
###############################################################################
sub dummy {
}

###############################################################################
&Initialise();

&loadScheduleFile();

	# Loop around processing each scheduled item as & when it's required .......
while (1) {
	&getNextItem();
	sleep($SleepTime);	# now pause before checking again
}
