#!/usr/local/bin/perl
###############################################################################
#		Written By:  Torquil Gault (peterb@hotmail.com) C) 2000-
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
###############################################################################

#
###############################################################################
sub verifyTime {
	my($h,$m) = split(":", $_[0]);		# break it up by it's usual delimiter
	if (! $m) {									# didnt work so try something else
		my($tm) = $_[0];
		$tm =~ s?\D??g;		# remove anything that is not a digit
		$tm = sprintf("%4.4d", $tm);
		$h = substr($tm, 0, 2);
		$m = substr($tm, 2);
	}
	return "$h:$m";
}

#
###############################################################################
sub verifyDate {
	my($d,$m,$y) = split($Date_Delimiters, $_[0]);
	if (! $d || ! $m || ! $y) {
		my($dt) = $_[0];
		$dt =~ s?\D??g;		# remove anything that is not a digit
		$d = substr($dt, 0, 2);
		$m = substr($dt, 2, 2);
		$y = substr($dt, 4);
	}
	return "$d/$m/$y";
}

#
###############################################################################
sub getCurrentDate {
	my($dt) = `date $Date_Fmt_2`;
	chop($dt);
	return $dt;
}

#
###############################################################################
sub getCurrentTime {
	my($dt) = `date $Time_Fmt_1`;
	chop($dt);
	return $dt;
}

#
###############################################################################
sub getDate {
	my($dt) = `date $Date_Fmt_1`;
	chop($dt);
	return $dt;
}

#
###############################################################################
sub setCurrentDate {
	($Day, $Month, $Year) = split($Date_Delimiters, &getCurrentDate());
	$curDate = join("", $Year, $Month, $Day);	# a var for easy checking
	dprint("scDate = $Day/$Month/$Year");
}

#
###############################################################################
sub setCurrentTime {
	($Hour, $Min) = split(":", &getCurrentTime());
	$curTime = join("", $Hour, $Min);	# a var for easy checking
	dprint("scTime = $Hour:$Min");
}

#
###############################################################################
@MonthDays = ([0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31],
				  [0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]);
@TotalDays = ([0, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365],
				  [0, 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366]);

@DaysFold = (6,0,1,2,3,4,5);

###############################################################################
#  Is the year passed to it a leap year ???
sub IsLeapYear {
	if ($_[0] % 4 == 0 && ($_[0] % 100 != 0 || $_[0] % 400 == 0)) {
		return $True;
	}
	return $False;
}

###############################################################################
# Converts a date to the no. of days since 1/1/0000. It can be called with
# "date" or dd,mm,yyyy
sub DateToDays {
	my $np = @_; 			# No of params
	my $d = $m = $y = 0;
	if ($np == 3) {		# Check the no of parameters passed
		($d, $m, $y) = @_;
	}
	elsif ($np == 1) {
		($d, $m, $y) = split($Date_Delimiters, $_[0]);
	}
	else {
		return 0;
	}
	my $total = 0;

	my $isLeap = &IsLeapYear($y);
	my $mon = $MonthDays[$isLeap][$m];
# print "$d/$m/$y : $isLeap : $mon : $m : $np : $_[0]\n";

	if ($d < 1 || $d > $mon) {		# Quick sanity check of the date
		return 0;
	}
	my $yr = int($y / 4);
	$total += $yr * 1461;
	$yr *= 4;
# print "Yr = $yr   Total = $total\n";
	while ($yr < $y) {
		$isLeap = &IsLeapYear($yr);
		$total += (365 + $isLeap);
		$yr++;
	}
	$isLeap = &IsLeapYear($y);
	$total += $TotalDays[$isLeap][$m];
	$total += $d;

	dprint("dateToDays: $d/$m/$y = $total");
	return $total;
}
	
###############################################################################
# Converts a Long to a date. Starts from 1/1/0000.
sub DaysToDate {
	my $total = $_[0];
	my $yr = int($total / 1461) * 4;
	my $daysleft = int($total % 1461);
	my $isLeap = &IsLeapYear($yr);
	if ($daysleft == 0 && $isLeap == 1) {
		$yr --;
		$isLeap = 0;
		$daysleft = 365;
	}
	my $daysayear = $isLeap + 365;
	while ($daysleft > $daysayear) {
		$yr ++;
		$daysleft -= $daysayear;
		$isLeap = &IsLeapYear($yr);
		$daysayear = $isLeap + 365;
	}
	my $mon = 1;
	while ($daysleft > $MonthDays[$isLeap][$mon]) {
		$daysleft -= $MonthDays[$isLeap][$mon];
		$mon ++;
	}
	dprint("daysToDate: $_[0] = $daysleft/$mon/$yr");
	return "$daysleft/$mon/$yr";
}

###############################################################################
# add/subtract days from a date. It can be called with "date", no_days or dd,mm,yy,no_days
sub dateMaths {
	my $np = @_;		# number of param's passed
	my $date = "";
	my $nd = 0;
	if ($np == 2) {	# assume date, no_days
		($date, $nd) = @_;
		dprint("dateMaths: $date, $nd");
		$nd = &DateToDays($date) + $nd;
	}
	elsif ($np == 4) {	# assume dd,mm,yy,no_days
		$nd = $_[3];
		dprint("dateMaths: $_[0]/$_[1]/$_[2], $nd");
		$nd = &DateToDays($_[0], $_[1], $_[2]) + $nd;
	}

	return &DaysToDate($nd);
}

#
###############################################################################
sub julianDate {			# Calculate the Julian Date of a date
	my($d, $m, $y) = @_;

	my $isLeap = &IsLeapYear($y);
	my $j = $TotalDays[$isLeap][$m]+$d;

	dprint("jD: $j ($d:$m:$y)");

	return $j;
}

#
###############################################################################
sub dayOfWeek {			# Calculate the day-of-the-week of a date
	my($d, $m, $y) = @_;

	my $dow = $y + (($y + 3) / 4);
   $dow -= ($y - 1701) / 100;
   $dow += ($y - 1601) / 400;
   $dow += &julianDate($d, $m, $y);
   $dow %= 7;

	dprint("dOW: $DaysFold[$dow]:($d:$m:$y)");

   return($DaysFold[$dow]);
}

#
###############################################################################
sub dummy {
}

return true;
